C     ********************************************************************************
C
C     subroutine CN_Method
C     Simulation of the catchment as impervious or simulation of a single rainfall 
C     event with the Curve number method
C
C     ********************************************************************************

      subroutine CN_method(PasPio,chkPL,PioN,STpn,INVpn)

      implicit none

      include 'dimarrays.inc'
      include 'comread.inc'
      include 'comrainrun.inc'
      include 'comvar.inc'

      integer PasPio,I,J,chkPL,STpn(Nrow,Ncol)  
      real Utempo,Pl,PioN(Nrow,Ncol),INVpn(Nrow*Nraing,Ncol)
      real QC0,QC1,PioL2,PN1,PN2

C              PioL2 : local rainfall at step t [mm]
C              PioL1 : local rainfall at step t-dt [mm]
C              PN2 : intensity of the rainfall that reaches the ground at step t[mm]
C              PN1 : intensity of the rainfall that reaches the ground at step t-dt [mm]
C              Pl  : local rainfall [mm] referred to the interval (t,t-dt) 
C              ES  : intensity of the infiltrated water [mm] 

      Utempo=DT/StepPio

      do I=1,idim
         do J=1,jdim
            if (Slope(I,J).GT.0) then
C              computation of the local rainfall
               call Local_rain(I,J,chkPL,Utempo,PasPio,Pl,STpn,INVpn)
               PioL2=PioL1(I,J)+Pl
               TotPio(I,J)=TotPio(I,J)+Pl ! total local rainfall
               PN2=(PioL2-cint*S(I,J))
               if (PN2.LT.0.) PN2=0.
               PN1=PioL1(I,J)-cint*S(I,J)
               if (PN1.LT.0) PN1=0.
               QC1=PN2**2/(PioL2+(1-cint)*S(I,J))
               if (QC1.LT.0.) QC1=0.
               QC0=PN1**2/(PioL1(I,J)+(1-cint)*S(I,J))
               if (QC0.LT.0.) QC0=0.
               PioL1(I,J)=PioL2
               PioN(I,J)=QC1-QC0
               if (PioN(I,J).LT.0.) PioN(I,J)=0.
               if (Pl.LE.0) then
                  ES(I,J)=0.
               else
                  ES(I,J)=Pl-PioN(I,J)
               end if
               if ((risp.EQ.2).AND.(risp1.EQ.1)) ES(I,J)=0.  ! simulation of impervious catchment
            else
               PioN(I,J)=-1.
            end if
         end do
      end do

      return
      end
