C     ************************************************************************************
C
C     subroutine Continuous
C     Continuous rainfall-runoff simulation. Conceptualisation of the interaction of 
C     soil, atmosphere and vegetation
C
C     ************************************************************************************

      subroutine Continuous(PasPio,K,Nstep,m,chkPL,chkTE,
     .                      STtn,PioN,STpn,INVpn)

      implicit none

      include 'dimarrays.inc'
      include 'comread.inc'
      include 'comrainrun.inc'
      include 'comvar.inc'

      real Utempo,Pl,Plt,Pn,TEM,QTEM,K1,K2,K3,K4,DF,Fprev
      real INVpn(Nrow*Nraing,Ncol),PioN(Nrow,Ncol)
      integer Nstep,chkTE,m,PasPio,I,J,K,chkPL
      integer STpn(NRow,NCol),STtn(Nrow,Ncol)


      Utempo=DT/StepPio

      I=1
      do while (I.LE.idim)
         J=1
         do while (J.LE.jdim)
            if (Slope(I,J).GT.0) then
C              computation of the local rainfall
               call Local_rain(I,J,chkPL,Utempo,PasPio,Pl,STpn,INVpn)
               TotPio(I,J)=TotPio(I,J)+Pl  ! total local rainfall [mm]
               Inter(I,J)=Inter(I,J)+Pl   ! Pl is initially intercepted by the vegetation [mm]
                                          ! (interception reservoir)
               if (Inter(I,J).GT.cint*S(I,J)) then  ! cint*S is the capacity of the interception reservoir [mm]
                  Pl=Inter(I,J)-cint*S(I,J)         ! Pl is the remaining rainfall that reaches the ground [mm]
                  Inter(I,J)=cint*S(I,J)     
               else
                  Pl=0.  
               end if

C              Resolution of the equation governing the soil water balance of the infiltration
C              reservoir with the fourth-order Runge-Kutta formula                       
               Plt=Pl/DT
               K1=DT*(-F(I,J)*(1/Hs+Plt/(H*S(I,J)))+Plt-ETp1(I,J)) 
               if ((F(I,J)+K1/2).LT.0) F(I,J)=-(K1/2)
               K2=DT*(-(F(I,J)+K1/2)*
     .             (1/Hs+Plt/(H*S(I,J)))+Plt-ETp1(I,J))
               if ((F(I,J)+K2/2).LT.0) F(I,J)=-(K2/2)
               K3=DT*(-(F(I,J)+K2/2)*
     .             (1/Hs+Plt/(H*S(I,J)))+Plt-ETp1(I,J))
               if ((F(I,J)+K3).LT.0) F(I,J)=-K3
               K4=DT*(-(F(I,J)+K3)*(1/Hs+Plt/(H*S(I,J)))+Plt-ETp1(I,J))
               DF=(K1+2*K2+2*K3+K4)/6
                 
               Fprev=F(I,J)
               F(I,J)=Fprev+DF 
               if (F(I,J).LT.0) F(I,J)=0
                    
               PioN(I,J)=(((F(I,J)+Fprev)/2)*Pl)/(H*S(I,J))     ! [mm]
               W(I,J)=(1/Hs)*((F(I,J)+Fprev)/2)*DX*DY/1000.     ! [m^3/s]

               if (PioN(I,J).GT.Pl) PioN(I,J)=Pl 
               if (F(I,J).GT.(H*S(I,J))) F(I,J)=H*S(I,J)  
   
C              computation of the evapotranspiration
               if (chkTE.EQ.0) then
                  TEM=gradi(PasPio,STt(I,J))
                  QTEM=alt(STt(I,J))
               else
                  TEM=gradi(PasPio,STtn(I,J))
                  QTEM=alt(STtn(I,J))
               end if
               TE(I,J)=TEM-0.0063*(QUO(I,J)-QTEM)
C              Computation of the potential evapotranspiration with the radiation method
               ETp(I,J)=(aE+bE*TE(I,J)*Nsol(m)*We(m))/
     .                             ((24./(StepPio/3600.))*n(m))
               ETp(I,J)=ETp(I,J)*DT/StepPio
               if (ETp(I,J).LT.0) ETp(I,J)=0

C              the evapotranspiration starts from the interception reservoir
               Inter(I,J)=Inter(I,J)-ETp(I,J)
               if (Inter(I,J).LT.0) then
C              and continues from the soil
                  ETp1(I,J)=-Inter(I,J)/DT 
                  Inter(I,J)=0.
               else
                  ETp1(I,J)=0. 
               end if 
            else
               PioN(I,J)=-1.
            end if
            J=J+1
         end do
         I=I+1
      end do

      return
      end